 
// File:    ARFFWriter.java
// Author:  Paul Marlow
// Date:    August 3, 2001
//

// Used for writing learning data out to file in ARFF format, so
// the file can be used as input into one of the Weka learning
// algorithms

import java.util.*;
import java.io.*;

public class ARFFWriter {
  private FileOutputStream output;
  private String filename;
  private String extension = ".arff";
  private String relation;
  private Vector attributes;
  private Vector data;

  public ARFFWriter()
  {
    output = null;
    filename = "";
    relation = "";
    attributes = new Vector();
    data = new Vector();
  }

  public ARFFWriter(String data_relation)
  {
    output = null;
    filename = "";
    relation = data_relation;
    attributes = new Vector();
    data = new Vector();
  }

  public String getRelation()
  {
    return relation;
  }

  public void setRelation(String data_relation)
  {
    relation = data_relation;
  }


  // ---------------------------- Methods for Attribute manipulation

  public void addAttribute(String attribute_name, String attribute_values)
  {
    String new_attribute = attribute_name + " " + attribute_values;
    attributes.add(new_attribute);
  }

  public int getAttributeSize()
  {
    return attributes.size();
  }

  public String getAttribute(int index)
  {
    return (String)(attributes.elementAt(index));
  }

  public void removeAttribute(int index)
  {
    attributes.removeElementAt(index);
  }

  public void clearAttributes()
  {
    attributes.clear();
  }


  // ---------------------------- Methods for Data manipulation

  public void addData(String training_data)
  {
    data.add(training_data);
  }

  public int getDataSize()
  {
    return data.size();
  }

  public String getData(int index)
  {
    return (String)(data.elementAt(index));
  }

  public void removeData(int index)
  {
    data.removeElementAt(index);
  }

  public void clearData()
  {
    data.clear();
  }

  // Checks whether all necessary information had been provided, and if
  // so, creates the ARFF file - with filename equal to the name of the
  // last attribute - since this is what will be learnt on by Weka.
  public void buildARFF()
  {
    if (checkProperties()) {
      String lastAttribute = (String)(attributes.lastElement());
      filename = lastAttribute.substring(0, lastAttribute.indexOf(' '));
      filename = filename + extension;
      String outline = "";

      try {
        output = new FileOutputStream(filename, false);

        try {
          // Write out relation to first line
          outline = "@relation " + relation;
          output.write((outline.trim() + '\n').getBytes());
          output.write((new String("\n")).getBytes());
          output.flush();

          // Write out attribute values
          for (int i = 0; i < attributes.size(); i++) {
            outline = "@attribute " + (String)(attributes.elementAt(i));
            output.write((outline.trim() + '\n').getBytes());
          }
          output.write((new String("\n")).getBytes());
          output.flush();

          // Write out training data
          outline = "@data";
          output.write((outline.trim() + '\n').getBytes());
          for (int j = 0; j < data.size(); j++) {
            outline = (String)(data.elementAt(j));
            output.write((outline.trim() + '\n').getBytes());
          }
          output.flush();

          // Close the file
          output.close();
        }
        catch (IOException ioe) {
          System.err.println("Error: Unable to write to file: " + filename);
        }
      }
      catch (IOException ioe) {
        System.err.println("Error: Unable to create file: " + filename);
      }
    }
  }


  // ---------------------------- PRIVATE METHODS

  private boolean checkProperties()
  {
    boolean valid = true;
    if (relation.compareTo("") == 0) {
      System.out.println("The Relation, i.e. one word description, must be supplied");
      valid = false;
    }
    if (attributes.size() < 2) {
      System.out.println("At least 2 attributes must be supplied - one input, and one for desired output");
      valid = false;
    }
    if (data.size() == 0) {
      System.out.println("There has been no data provided to train on.");
      valid = false;
    }
    return valid;
  }
}
